package com.sun.xml.internal.org.jvnet.mimepull;

import java.util.concurrent.Executor;
import java.util.ArrayList;
import java.util.logging.Level;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.File;
import java.util.List;
import java.lang.ref.ReferenceQueue;
import java.util.logging.Logger;
import java.lang.ref.WeakReference;

final class WeakDataFile extends WeakReference<DataFile>
{
    private static final Logger LOGGER;
    private static ReferenceQueue<DataFile> refQueue;
    private static List<WeakDataFile> refList;
    private final File file;
    private final RandomAccessFile raf;
    private static boolean hasCleanUpExecutor;
    
    WeakDataFile(DataFile df, File file) {
        super(df, WeakDataFile.refQueue);
        WeakDataFile.refList.add(this);
        this.file = file;
        try {
            this.raf = new RandomAccessFile(file, "rw");
        }
        catch (IOException ioe) {
            throw new MIMEParsingException(ioe);
        }
        if (!WeakDataFile.hasCleanUpExecutor) {
            drainRefQueueBounded();
        }
    }
    
    synchronized void read(long pointer, byte[] buf, int offset, int length) {
        try {
            this.raf.seek(pointer);
            this.raf.readFully(buf, offset, length);
        }
        catch (IOException ioe) {
            throw new MIMEParsingException(ioe);
        }
    }
    
    synchronized long writeTo(long pointer, byte[] data, int offset, int length) {
        try {
            this.raf.seek(pointer);
            this.raf.write(data, offset, length);
            return this.raf.getFilePointer();
        }
        catch (IOException ioe) {
            throw new MIMEParsingException(ioe);
        }
    }
    
    void close() {
        if (WeakDataFile.LOGGER.isLoggable(Level.FINE)) {
            WeakDataFile.LOGGER.log(Level.FINE, "Deleting file = {0}", this.file.getName());
        }
        WeakDataFile.refList.remove(this);
        try {
            this.raf.close();
            boolean deleted = this.file.delete();
            if (!deleted && WeakDataFile.LOGGER.isLoggable(Level.INFO)) {
                WeakDataFile.LOGGER.log(Level.INFO, "File {0} was not deleted", this.file.getAbsolutePath());
            }
        }
        catch (IOException ioe) {
            throw new MIMEParsingException(ioe);
        }
    }
    
    void renameTo(File f) {
        if (WeakDataFile.LOGGER.isLoggable(Level.FINE)) {
            WeakDataFile.LOGGER.log(Level.FINE, "Moving file={0} to={1}", new Object[] { this.file, f });
        }
        WeakDataFile.refList.remove(this);
        try {
            this.raf.close();
            boolean renamed = this.file.renameTo(f);
            if (!renamed && WeakDataFile.LOGGER.isLoggable(Level.INFO)) {
                WeakDataFile.LOGGER.log(Level.INFO, "File {0} was not moved to {1}", new Object[] { this.file.getAbsolutePath(), f.getAbsolutePath() });
            }
        }
        catch (IOException ioe) {
            throw new MIMEParsingException(ioe);
        }
    }
    
    static void drainRefQueueBounded() {
        WeakDataFile weak;
        while ((weak = (WeakDataFile)WeakDataFile.refQueue.poll()) != null) {
            if (WeakDataFile.LOGGER.isLoggable(Level.FINE)) {
                WeakDataFile.LOGGER.log(Level.FINE, "Cleaning file = {0} from reference queue.", weak.file);
            }
            weak.close();
        }
    }
    
    static {
        LOGGER = Logger.getLogger(WeakDataFile.class.getName());
        WeakDataFile.refQueue = new ReferenceQueue();
        WeakDataFile.refList = new ArrayList();
        WeakDataFile.hasCleanUpExecutor = false;
        CleanUpExecutorFactory executorFactory = CleanUpExecutorFactory.newInstance();
        if (executorFactory != null) {
            if (WeakDataFile.LOGGER.isLoggable(Level.FINE)) {
                WeakDataFile.LOGGER.log(Level.FINE, "Initializing clean up executor for MIMEPULL: {0}", executorFactory.getClass().getName());
            }
            Executor executor = executorFactory.getExecutor();
            executor.execute(new Runnable() {
                @Override
                public void run() {
                    while (true) {
                        try {
                            WeakDataFile weak = (WeakDataFile)WeakDataFile.refQueue.remove();
                            if (WeakDataFile.LOGGER.isLoggable(Level.FINE)) {
                                WeakDataFile.LOGGER.log(Level.FINE, "Cleaning file = {0} from reference queue.", weak.file);
                            }
                            weak.close();
                            continue;
                        }
                        catch (InterruptedException ex) {}
                    }
                }
            });
            WeakDataFile.hasCleanUpExecutor = true;
        }
    }
}
